/*
Проба токового регистора MBI5167
Динамическая индикация на четыре разряда 7-сегм. индикаторов
Выводы MBI5167 OUT0-OUT6 подключить к катодам a-g всех индикаторов,
вывод OUT7 - к десятичной точке DP
выводы 2-5 Arduino управляют разрядами 1-4
Демо-программа показывает цифру в разряде, равную номеру разряда - 1, т.е. 0123
Для засветки десятичной точки 
перед выводом добавить единицу к выбранному коду
*/

#define dataPin  10  // пин подключен к входу SDI
#define latchPin 11  // пин подключен к входу LE
#define clockPin 12  // пин подключен к входу CLK

#define Razr1 2  // разряд 1, включение =1 
#define Razr2 3  // разряд 2, включение =1 
#define Razr3 4  // разряд 3, включение =1 
#define Razr4 5  // разряд 4, включение =1 

byte code7seg [10] = {
  0b11111100, //0
  0b01100000, //1 
  0b11011010, //2 
  0b11110010, //3 
  0b01100110, //4 
  0b10110110, //5 
  0b10111110, //6 
  0b11100000, //7 
  0b11111110, //8
  0b11110110, //9 
};

byte code;
byte digit;
volatile byte iCount = 0; //счетчик прерываний

void setup() {
   // устанавливаем режим работы выводов OUTPUT
   pinMode(latchPin, OUTPUT);
   pinMode(clockPin, OUTPUT);
   pinMode(dataPin, OUTPUT);
   pinMode(Razr1, OUTPUT);
   pinMode(Razr2, OUTPUT);
   pinMode(Razr3, OUTPUT);
   pinMode(Razr4, OUTPUT);
  
  // инициализация Timer2 100 Гц
  TCCR2A = 0;
  TCCR2B = 0;
  OCR2A = 64; // установка регистра совпадения 4 мс (244 Гц)
  TCCR2A = (1 << WGM21);  // CTC режим
  TIMSK2 |= (1 << OCIE2A); // включение прерываний по совпадению
  TCCR2B = (1 << CS22) | (1 << CS21) | (1 << CS20); // запуск с таймера с делителем на 1024

}

ISR (TIMER2_COMPA_vect) { // прерывание 100 Гц
//гасим все
      digitalWrite(Razr1,LOW);  
      digitalWrite(Razr2,LOW); 
      digitalWrite(Razr3,LOW);
      digitalWrite(Razr4,LOW);
//переключаем цифру и зажигаем нужный разряд
switch (iCount) {
    case 0: //первый разряд
      digit=iCount; //здесь задаем значение цифры разряда 1 - для примера в данном случае равное iCount
      code = code7seg[digit]; // получаем код цифры
      shiftOut(dataPin, clockPin, LSBFIRST, code); // код цифры - в регистр
   digitalWrite(latchPin, HIGH); //переписываем а защелку
   digitalWrite(latchPin, LOW);
      digitalWrite(Razr1,HIGH);  //зажигаем 1 разряд
      break;
    case 1: //второй разряд
      digit=iCount; //здесь задаем значение цифры разряда 2 
      code = code7seg[digit]; // получаем код цифры
      shiftOut(dataPin, clockPin, LSBFIRST, code); // код цифры - в регистр
   digitalWrite(latchPin, HIGH); //переписываем а защелку
   digitalWrite(latchPin, LOW);
      digitalWrite(Razr2,HIGH); //зажигаем 2 разряд
      break;
    case 2: //третий разряд
      digit=iCount; //здесь задаем значение цифры разряда 3 
      code = code7seg[digit]; // получаем код цифры
      shiftOut(dataPin, clockPin, LSBFIRST, code); // код цифры - в регистр
   digitalWrite(latchPin, HIGH); //переписываем а защелку
   digitalWrite(latchPin, LOW);
      digitalWrite(Razr3,HIGH); //зажигаем 3 разряд
      break;
    case 3: //четвертый разряд
       digit=iCount; //здесь задаем значение цифры разряда 4 
      code = code7seg[digit]; // получаем код цифры
      shiftOut(dataPin, clockPin, LSBFIRST, code); // код цифры - в регистр
   digitalWrite(latchPin, HIGH); //переписываем а защелку
   digitalWrite(latchPin, LOW);
      digitalWrite(Razr4,HIGH); //зажигаем 4 разряд
      break;
  }  
  iCount++;
  if (iCount == 4) iCount=0;
}

void loop() {
 //основной цикл в примере не нужен, все в прерывании
}
